@@ -12,6 +12,21 @@ class DjangoPushDataHandler(object):
             self.model_map = self._build_model_map(models)
         self.model_field_map = dict()
 
+    def add_model_config(self, model, primary=None, fields=None, exclude_fields=None,table=None):
+        if not primary:
+            primary = ['id']
+        config = {
+            'model': model,
+            'primary': primary,
+        }
+        if fields:
+            config['fields'] = fields
+        if exclude_fields:
+            config['exclude_fields'] = exclude_fields
+        if not table:
+            table = model._meta.db_table
+        self.model_map[table] = config
+
     def _build_model_map(self, models):
         result = dict()
         for model in models:
@@ -59,15 +74,29 @@ class DjangoPushDataHandler(object):
         self._insert0(database, table, data, "insert")
         self.callback_handle(PUSH_ACTION_INSERT, database, table, data)
 
+    def before_update(self, database, table, data):
+        return True
+
     def _insert0(self, database, table, data, action):
         try:
             model_item = self._get_model(database, table)
             if not model_item:
                 return
+            after = data.get('after')
+            if after:
+                data = after
+            if not self.before_update(database, table, data):
+                return
             model = model_item['model']
             fields = model_item.get('fields')
             if not fields:
                 fields = self.get_model_fields(model)
+            else:
+                fields = set(fields)
+            exclude_fields = model_item.get('exclude_fields')
+            if exclude_fields:
+                for field in exclude_fields:
+                    fields.discard(field)
             data2 = self._build_update_data(data, model_item, fields)
             model.objects.update_or_create(**data2)
             sd.log_info(action + " data table={0} data={1}".format(table, data2))
@@ -83,9 +112,7 @@ class DjangoPushDataHandler(object):
         self.callback_handle(PUSH_ACTION_UPDATE, database, table, data)
 
     def _build_update_data(self, data, model_item, fields):
-        after = data.get('after')
-        if after:
-            data = after
+
         res = self.build_primary_keys(data, model_item['primary'])
         if not res:
             sd.log_error("build_primary_keys failed for data={0} primary={1}".format(data, model_item['primary']))
@@ -115,10 +142,15 @@ class DjangoPushDataHandler(object):
             res[key] = val
         return res
 
+    def before_delete(self, database, table, data):
+        return True
+
     def delete(self, database, table, data):
         model_item = self._get_model(database, table)
         if not model_item:
             return
+        if not self.before_delete(database, table, data):
+            return
         model = model_item['model']
         res = self.build_primary_keys(data, model_item['primary'])
         if not res:
